import click

import requests
from .ChatGPT import Chatbot
import os
import sys
from rich.live import Live
from rich.markdown import Markdown

sys.stdin.reconfigure(encoding='utf-8')

@click.command()
def hello():
    """Say hello"""
    click.echo('Hello, world!')

@click.group()
def dxr():
    """Figlu command line tool"""
    # 默认调用bash函数
    pass

def get_bash_scripts():
    url = 'http://39.101.69.111:4000/bash'
    r = requests.get(url)
    scripts = r.json()
    return scripts

def get_bash_script_dir(script):
    url = f'http://39.101.69.111:4000/bash/{script}'
    r = requests.get(url)
    scripts = r.json()
    return scripts

def get_bash_script(script, sub_script):
    url = f'http://39.101.69.111:4000/bash/{script}/{sub_script}'
    r = requests.get(url)
    script = r.text
    return script

def choose_script(scripts, step=1):
    script_index = 0
    while True:
        click.clear()
        click.echo('Select a script:')
        for i, script in enumerate(scripts):
            if i == script_index:
                click.echo(f'> {script}')
            else:
                click.echo(f'  {script}')
        key = click.getchar()
        # 上下键选择，左键返回，右键进入
        if key == '\r':
            return scripts[script_index], step + 1
        elif key == '\x1b[A':
            script_index = (script_index - 1) % len(scripts)
        elif key == '\x1b[B':
            script_index = (script_index + 1) % len(scripts)
        elif key == '\x1b[D':
            return None, step - 1
        elif key == '\x1b[C':
            return scripts[script_index], step + 1
            

@dxr.command()
def bash():
    """Bash scripts"""
    import os
    # scripts = get_bash_scripts()
    # script = choose_script(scripts)
    # sub_scripts = get_bash_script_dir(script)
    # sub_script = choose_script(sub_scripts)
    # bash_script = get_bash_script(script, sub_script)
    # print("=" * 80)
    # click.echo(bash_script)
    # print("=" * 80)
    # click.echo('Run this script?')
    # if click.confirm('Continue?'):
    #     os.system(bash_script)
    step = 1
    while True:
        if step == 0:
            break
        if step == 1:
            scripts = get_bash_scripts()
            script, step = choose_script(scripts, step)
        elif step == 2:
            sub_scripts = get_bash_script_dir(script)
            sub_script, step = choose_script(sub_scripts, step)
        elif step == 3:
            bash_script = get_bash_script(script, sub_script)
            print("=" * 80)
            click.echo(bash_script)
            print("=" * 80)
            click.echo('Run this script?')
            if click.confirm('Continue?'):
                os.system(bash_script)
                break
            step = 2
    
def get_api_key():
    api_key = os.environ.get("OPENAI_API_KEY", '')
    if api_key == '':
        url = f'http://39.101.69.111:4000/chat/get_api_key'
        r = requests.get(url)
        base_url_and_api_key = r.text
        base_url, api_key = base_url_and_api_key.split(',')
    return base_url, api_key  
    
@click.command()
def chat():
    """Chat with GPT-3"""
    base_url, api_key = get_api_key()
    bot = Chatbot(api_key=api_key, base_url=base_url)
    while True:
        text = input("You: ")
        if text.strip() == "exit":
            break
        response = bot.ask_stream(text)
        md = Markdown("")
        with Live(md, auto_refresh=False) as live:
            tmp = ""
            for r in response:
                tmp += r
                md = Markdown(tmp)
                live.update(md, refresh=True)
                

@click.command()
def git():
    """Git scripts"""
    print("这是一个git命令行助手,你可以通过这个助手来学习git命令")
    # use openai to generate git scripts
    base_url, api_key = get_api_key()
    bot = Chatbot(api_key=api_key, base_url=base_url, system_prompt="""
    你是一个很好的git命令行助手，你可以帮助我更好的使用git, 
    根据我的问题,你可以给我提供一些git命令
    """)
    while True:
        text = input("请输入你的问题: ")
        if text.strip() == "exit":
            break
        response = bot.ask_stream(text)
        print("=" * 80)
        tmp = ""
        for r in response:
            tmp += r
            print(r, end='', flush=True)
        print()
        print("=" * 80)
    
    
   
    


def main():
    print("Hello world!")

dxr.add_command(hello)
dxr.add_command(bash)
dxr.add_command(chat)
dxr.add_command(git)

if __name__ == '__main__':
    dxr()



"""
A simple wrapper for the official ChatGPT API
"""
import argparse
import json
import os
import sys
from typing import NoReturn

import requests
import tiktoken

from .utils import create_completer
from .utils import create_session
from .utils import get_input
import openai

ENGINE = os.environ.get("GPT_ENGINE") or "gpt-3.5-turbo"


class Chatbot:
    """
    Official ChatGPT API
    """

    def __init__(
        self,
        api_key: str,
        base_url: str = None,
        engine: str = None,
        proxy: str = None,
        max_tokens: int = 4095,
        temperature: float = 0.5,
        top_p: float = 1.0,
        reply_count: int = 1,
        system_prompt: str = "You are ChatGPT, a large language model trained by OpenAI. Respond conversationally",
    ) -> None:
        """
        Initialize Chatbot with API key (from https://platform.openai.com/account/api-keys)
        """
        self.engine = engine or ENGINE
        self.session = requests.Session()
        self.api_key = api_key
        openai.api_key = self.api_key
        if base_url is not None:
            openai.api_base = base_url + '/v1'
        self.proxy = proxy
        if self.proxy:
            proxies = {
                "http": self.proxy,
                "https": self.proxy,
            }
            self.session.proxies = proxies
        self.conversation: dict = {
            "default": [
                {
                    "role": "system",
                    "content": system_prompt,
                },
            ],
        }
        self.system_prompt = system_prompt
        self.max_tokens = max_tokens
        self.temperature = temperature
        self.top_p = top_p
        self.reply_count = reply_count

        if self.get_token_count("default") > self.max_tokens:
            raise Exception("System prompt is too long")

    def add_to_conversation(
        self,
        message: str,
        role: str,
        convo_id: str = "default",
    ) -> None:
        """
        Add a message to the conversation
        """
        self.conversation[convo_id].append({"role": role, "content": message})

    def __truncate_conversation(self, convo_id: str = "default") -> None:
        """
        Truncate the conversation
        """
        while True:
            if (
                self.get_token_count(convo_id) > self.max_tokens
                and len(self.conversation[convo_id]) > 1
            ):
                # Don't remove the first message
                self.conversation[convo_id].pop(1)
            else:
                break

    # https://github.com/openai/openai-cookbook/blob/main/examples/How_to_count_tokens_with_tiktoken.ipynb
    def get_token_count(self, convo_id: str = "default") -> int:
        """
        Get token count
        """
        if self.engine not in ["gpt-3.5-turbo", "gpt-3.5-turbo-0301", 'gpt-4-0314', 'gpt-4']:
            raise NotImplementedError("Unsupported engine {self.engine}")

        encoding = tiktoken.encoding_for_model(self.engine)

        num_tokens = 0
        for message in self.conversation[convo_id]:
            # every message follows <im_start>{role/name}\n{content}<im_end>\n
            num_tokens += 4
            for key, value in message.items():
                num_tokens += len(encoding.encode(value))
                if key == "name":  # if there's a name, the role is omitted
                    num_tokens += -1  # role is always required and always 1 token
        num_tokens += 2  # every reply is primed with <im_start>assistant
        return num_tokens

    def get_max_tokens(self, convo_id: str) -> int:
        """
        Get max tokens
        """
        return 4000 - self.get_token_count(convo_id)

    def ask_stream(
        self,
        prompt: str,
        role: str = "user",
        convo_id: str = "default",
        **kwargs,
    ) -> str:
        """
        Ask a question
        """
        # Make conversation if it doesn't exist
        if convo_id not in self.conversation:
            self.reset(convo_id=convo_id, system_prompt=self.system_prompt)
        self.add_to_conversation(prompt, "user", convo_id=convo_id)
        self.__truncate_conversation(convo_id=convo_id)
        response = openai.ChatCompletion.create(
            model=self.engine,
            messages=self.conversation[convo_id],
            temperature=kwargs.get("temperature", self.temperature),
            top_p=kwargs.get("top_p", self.top_p),
            n=kwargs.get("n", self.reply_count),
            stream=True,
            user=role,
        )  
        response_role: str = None
        full_response: str = ""
        for resp in response:
            choices = resp.get("choices")
            if not choices:
                continue
            delta = choices[0].get("delta")
            if not delta:
                continue
            if "role" in delta:
                response_role = delta["role"]
            if "content" in delta:
                content = delta["content"]
                full_response += content
                yield content
        self.add_to_conversation(full_response, response_role, convo_id=convo_id)

    def ask(
        self,
        prompt: str,
        role: str = "user",
        convo_id: str = "default",
        **kwargs,
    ) -> str:
        """
        Non-streaming ask
        """
        response = self.ask_stream(
            prompt=prompt,
            role=role,
            convo_id=convo_id,
            **kwargs,
        )
        full_response: str = "".join(response)
        return full_response

    def rollback(self, n: int = 1, convo_id: str = "default") -> None:
        """
        Rollback the conversation
        """
        for _ in range(n):
            self.conversation[convo_id].pop()

    def reset(self, convo_id: str = "default", system_prompt: str = None) -> None:
        """
        Reset the conversation
        """
        self.conversation[convo_id] = [
            {"role": "system", "content": system_prompt or self.system_prompt},
        ]

    def save(self, file: str, *convo_ids: str) -> bool:
        """
        Save the conversation to a JSON file
        """
        try:
            with open(file, "w", encoding="utf-8") as f:
                if convo_ids:
                    json.dump({k: self.conversation[k] for k in convo_ids}, f, indent=2)
                else:
                    for convo_id in self.conversation:
                        system_message = self.conversation[convo_id][0]
                        # 只保留50条对话，但是必须要保留系统的第一条消息
                        self.conversation[convo_id] = self.conversation[convo_id][-50:]
                        self.conversation[convo_id][0] = system_message
                    json.dump(self.conversation, f, indent=2)
        except (FileNotFoundError, KeyError):
            return False
        return True
        # print(f"Error: {file} could not be created")

    def load(self, file: str, *convo_ids: str) -> bool:
        """
        Load the conversation from a JSON  file
        """
        try:
            with open(file, encoding="utf-8") as f:
                if convo_ids:
                    convos = json.load(f)
                    self.conversation.update({k: convos[k] for k in convo_ids})
                else:
                    self.conversation = json.load(f)
        except (FileNotFoundError, KeyError, json.decoder.JSONDecodeError):
            return False
        return True
    
    def voice_2_text(self, file):
        # 判断file是文件还是文件的字符串，如果是文件的字符串，就转换成文件
        if isinstance(file, str):
            file = open(file, 'rb')
        response = openai.Audio.transcribe('whisper-1', file)
        return response['text']


from concurrent import futures
import copy
import time
import cv2
import grpc
import numpy as np
from . import Datas_pb2
from . import Datas_pb2_grpc
import json


class Greeter(Datas_pb2_grpc.MainServerServicer):
    def __init__(self, rec_callback=None):
        self.rec_callback = rec_callback
    
    def getStream(self, request_iterator, context):
        for request in request_iterator:
            dBuf = np.frombuffer(request.datas, dtype=np.uint8)
            dst = cv2.imdecode(dBuf, cv2.IMREAD_COLOR)
            dst_copy = copy.deepcopy(dst)
            json_data = json.loads(request.json_data)
            if self.rec_callback:
                dst, json_data = self.rec_callback(dst, json_data)
            if dst is None:
                dst = dst_copy
            _, buf = cv2.imencode('.jpg', dst)
            yield Datas_pb2.Reply(datas = buf.tobytes(), json_data = json.dumps(json_data))



class Dxr_grpc_server:
    def __init__(self, port=50051, rec_callback=None) -> None:
        self.port = port
        self.rec_callback = rec_callback

    def serve(self):
        server = grpc.server(futures.ThreadPoolExecutor(max_workers=10))
        Datas_pb2_grpc.add_MainServerServicer_to_server(Greeter(self.rec_callback), server)
        server.add_insecure_port('[::]:'+str(self.port))
        server.start()
        print('server start')
        try:
            while True:
                time.sleep(1/60)

        except KeyboardInterrupt:
            server.stop(0)
            
    def run(self):
        self.serve()


import os
import stat
import threading
import time
import requests


def ssh_download_file(remote_path, local_path, ip, user_name, password, port=22, callback=None):
    # 使用requests下载文件,并调用callback函数,持续返回进度
    url = f'http://{ip}:{port}/download_file?remote_path={remote_path}'
    print(url)
    # 获取文件的大小
    response = requests.get(url=url, auth=(user_name, password), stream=True)
    response.raise_for_status()
    file_size = int(response.headers['Content-Length'])
    # 获取文件的名字
    file_name = os.path.basename(remote_path)
    local_file_path = os.path.join(local_path, file_name)
    # 判断文件是否存在
    if os.path.isfile(local_file_path):
        # 如果本地文件已经存在，则删除
        os.remove(local_file_path)
        time.sleep(1)
    # 下载文件
    response = requests.get(url=url, auth=(user_name, password), stream=True)
    response.raise_for_status()
    # 保存文件
    with open(local_file_path, 'wb') as f:
        # 记录下载的大小
        count = 0
        # 记录下载的时间
        start_time = time.time()
        # 以流的形式下载文件
        process = 0
        for chunk in response.iter_content(chunk_size=1024):
            if chunk:
                f.write(chunk)
                count += len(chunk)
                # 每下载1M,调用一次callback函数
                if count / file_size > 0.01:
                    # 调用callback函数
                    if callback:
                        process += count / file_size
                        # threading.Thread(target=callback, args=(round(process * 100, 2), file_name)).start()
                        callback(round(process * 100, 2), file_name)
                    count = 0
        # threading.Thread(target=callback, args=(100, file_name)).start()
        callback(100, file_name)
                    
    print('下载完成')
    return True
    
def callback(process, filename):
    # 使用锁来确保线程安全
    with threading.Lock():
        print(process, filename)
    
    
if __name__ == '__main__':
    remote_path = '/root/nav_ws/src/lidar_localization/Localization/data/latest/finalCloud.pcd'
    local_path = './'
    ip = '10.10.9.254'
    user_name = 'root'
    password = '0'
    port = 5000
    res = ssh_download_file(remote_path, local_path, ip, user_name, password, port, callback)



import queue
import threading
# from Dxr_mqtt.dxr_log import *
import asyncio
import logging
import time
import cv2
import numpy as np
from aiortsp.transport import RTPTransportClient, transport_for_scheme
from aiortsp.rtsp.connection import RTSPConnection
from aiortsp.rtsp.session import RTSPMediaSession
logger = logging.getLogger('rtsp_client')
logging.basicConfig(format='%(asctime)s %(levelname)s %(message)s')
logger.setLevel(logging.ERROR)

recv = b''
thermalStream = b''
frame_list = []

class Probe(RTPTransportClient):

    def __init__(self, origin_queue=None):
        self.rtp_count = 0
        self.rtcp_count = 0
        self.origin_queue = origin_queue

    def handle_rtp(self, rtp):
        global thermalStream, frame_list
        # V：RTP协议的版本号，占2位，当前协议版本号为2
        # P：填充标志，占1位，如果P=1，则在该报文的尾部填充一个或多个额外的八位组，它们不是有效载荷的一部分
        # X：扩展标志，保留为0，后续如有需求可增加扩展头信息
        # CC：CSRC计数器，占4位，指示CSRC标识符的个数
        # M: maker标志位； thermalStream数据较多时将采取分包发送，分包数据maker标志位为0，结束包maker标志位为 1
        # PT: 有效载荷类型，占7位，用于说明RTP报文中有效载荷的类型，当前热成像流统一返回 109（即0x6D
        # 序列号：占16位，用于标识发送者所发送的RTP报文的序列号，每发送一个报文，序列号增1。这个字段当下层的 承载协议用UDP的时候，网络状况不好的时候可以用来检查丢包。同时出现网络抖动的情况可以用来对数据进行 重新排序，在helix服务器中这个字段是从0开始的，同时音频包和视频包的sequence是分别记数的；
        # 时戳(Timestamp)：占32位，时戳反映了该RTP报文的第一个八位组的采样时刻。接收者使用时戳来计算延迟和延 迟抖动，并进行同步控制
        # 同步信源(SSRC)标识符：占32位，用于标识同步信源。该标识符是随机选择的，参加同一视频会议的两个同步信 源不能有相同的SSRC
        # 特约信源(CSRC)标识符：每个CSRC标识符占32位，可以有0～15个。每个CSRC标识了包含在该RTP报文有效载荷中 的所有特约信源
        # 载荷 encoding 为thermalStream 类型
        # FUA：FU Indicator，FU Header，FU Payload
        # FU Indicator：FU Indicator是一个8bit的字段，它的结构如下：
        # 0 1 2 3 4 5 6 7
        # +---------------+
        # |0|1|0|0|0|0|0|0|
        # +---------------+
        # |F|NRI|  Type   |
        # +---------------+
        # F：0表示单一NALU，1表示分片NALU
        # NRI：NALU的重要性指示，值越大，重要性越高
        # Type：NALU的类型, 16~23为FU-A，24~31为FU-B
        # 前两位为版本号，后面为负载类型
        # FU Header：FU Header是一个8bit的字段，它的结构如下：
        # 0 1 2 3 4 5 6 7
        # +---------------+
        # |0|1|0|0|0|0|0|0|
        # +---------------+
        # |S|E|R|  Type   |
        # +---------------+
        # S：1表示该NALU的开始，0表示该NALU的中间或结束
        # E：1表示该NALU的结束，0表示该NALU的中间或开始
        # R：保留位
        # Type：NALU的类型, 16~23为FU-A，24~31为FU-B
        d = rtp.pack()
        v = d[0] >> 6
        p = (d[0] >> 5) & 0x01
        x = (d[0] >> 4) & 0x01
        cc = d[0] & 0x0f
        m = (d[1] >> 7) & 0x01
        pt = d[1] & 0x7f
        seq = (d[2] << 8) + d[3]
        ts = (d[4] << 24) + (d[5] << 16) + (d[6] << 8) + d[7]
        ssrc = (d[8] << 24) + (d[9] << 16) + (d[10] << 8) + d[11]
        # 特约信源(CSRC)标识符：每个CSRC标识符占32位，可以有0～15个。每个CSRC标识了包含在该RTP报文有效载荷中 的所有特约信源
        csrc = []
        for i in range(cc):
            csrc.append((d[12 + i * 4] << 24) + (d[13 + i * 4] << 16) + (d[14 + i * 4] << 8) + d[15 + i * 4])
        payload = d[12 + cc * 4:]
        # 如果P=1，则在该报文的尾部填充一个或多个额外的八位组，它们不是有效载荷的一部分
        if p == 1:
            payload = payload[:-1]
        if m == 1:
            thermalStream += payload
            
            # 如果长度不为442500，如果大于442500，截取442500长度的数据，如果小于442500，将数据拼接到thermalStream中
            if len(thermalStream) == 442500:
                # print('v:{}, p:{}, x:{}, cc:{}, m:{}, spt:{}, seq:{}, ts:{}, ssrc:{}, csrc:{}, len(thermalStream):{}'.format(v, p, x, cc, m, pt, seq, ts, ssrc, csrc, len(thermalStream)), end='\r')  
                # 去掉前128 + 4个字节，剩下的为384*288*4 表示每个像素点的温度数据用 float（4 字节）
                frame = np.frombuffer(thermalStream[132:], dtype=np.float32).reshape(288, 384)
                if self.origin_queue is not None:
                    if self.origin_queue.full():
                        self.origin_queue.get()
                    self.origin_queue.put(frame)
            else:
                # print(len(thermalStream))
                pass
            thermalStream = b''
        else:
            thermalStream += payload  
        # payload为视频数据，可以进行解码，转码，保存等操作
        # print(payload)
        self.rtp_count += 1

    def handle_rtcp(self, rtcp):
        self.rtcp_count += 1
        logger.debug('RTCP received: %s', rtcp)

class RTSP_Client:
    def __init__(self, ip, port, username, password, logger=None):
        self.ip = ip
        self.port = port
        self.username = username
        self.password = password
        self.url = f'rtsp://{username}:{password}@{ip}:{port}/ISAPI/Streaming/thermal/channels/2/streamType/pixel-to-pixel_thermometry_data'
        print(self.url)
        self.origin_queue = queue.Queue(maxsize=1)
        self.probe = Probe(self.origin_queue)
        # 在这里启动一个线程，用来接收视频数据
        self.thread = threading.Thread(target=self.start)
        self.thread.setDaemon(True)
        self.is_running = True
        self.thread.start()
        self.logger = logger
        
    def start(self):
        loop = asyncio.new_event_loop()
        asyncio.set_event_loop(loop)
        loop.run_until_complete(self.connect())
        
    def stop(self):
        self.is_running = False
        self.thread.join()

    async def connect(self):
        async  with RTSPConnection(self.ip, self.port, self.username, self.password, logger=None) as conn:
            transport_class = transport_for_scheme('rtsp')
            async with transport_class(conn, logger=None, timeout=0) as transport:
                transport.subscribe(self.probe)
                async with RTSPMediaSession(conn, self.url, transport, logger=self.logger, media_type='application') as sess:
                    try:
                        print('starting stream...')
                        await sess.play()
                        print('stream started')
                        while conn.running and transport.running:
                            print('running...')
                            await asyncio.sleep(sess.session_keepalive)
                            await sess.keep_alive()
                    except asyncio.CancelledError:
                        print('stopping stream...')
                        #断线重连
            await sess.teardown()
                        
    def read(self):
        return time.time(), self.origin_queue.get()

    # 定义一个方法，用来将温度数据转换为frame
    def transform(self, frame):
        frame_original = cv2.normalize(frame, None, 0, 255, cv2.NORM_MINMAX, cv2.CV_8U)
        frame_original = cv2.cvtColor(frame_original, cv2.COLOR_GRAY2BGR)
        frame_original = cv2.applyColorMap(frame_original, cv2.COLORMAP_JET)
        return frame_original
    
    # 定义一个方法用来获取温度大于某个值的坐标
    def get_point(self, frame, threshold):
        point_list = []
        for i in range(frame.shape[0]):
            for j in range(frame.shape[1]):
                if frame[i, j] > threshold:
                    point_list.append((j, i))
        return point_list

if __name__ == '__main__':
    # asyncio.run(main())
    # logger = logging.getLogger('rtsp')
    # logger.setLevel(logging.DEBUG)
    client = RTSP_Client('192.168.3.68', 554, 'admin', 'Asb11023', logger=None)
    while True:
        time_stamp, temp_data = client.read()
        print(time_stamp)
        frame = client.transform(temp_data)
        # print(client.get_point(temp_data, 10))
        cv2.imshow('frame', frame)
        cv2.waitKey(1)


# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
import grpc

from . import Datas_pb2 as Datas__pb2


# import Datas_pb2 as Datas__pb2


class MainServerStub(object):
    """responce server
  """

    def __init__(self, channel):
        """Constructor.

    Args:
      channel: A grpc.Channel.
    """
        self.getStream = channel.stream_stream(
            '/MainServer/getStream',
            request_serializer=Datas__pb2.Request.SerializeToString,
            response_deserializer=Datas__pb2.Reply.FromString,
        )


class MainServerServicer(object):
    """responce server
  """

    def getStream(self, request_iterator, context):
        # missing associated documentation comment in .proto file
        pass
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_MainServerServicer_to_server(servicer, server):
    rpc_method_handlers = {
        'getStream': grpc.stream_stream_rpc_method_handler(
            servicer.getStream,
            request_deserializer=Datas__pb2.Request.FromString,
            response_serializer=Datas__pb2.Reply.SerializeToString,
        ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
        'MainServer', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))



import asyncio
import os
import stat
import types
import re
from Dxr_mqtt.dxr_log import *


# 判断系统类型，如果是windows则引入wexpect，如果是linux则引入pexpect
if os.name == 'nt':
    import wexpect as pexpect
else:
    import pexpect
password_key = '.*assword.*'
is_input_yes = '.*yes/no.*'
import threading
import time

# 写一个线程，一直来对比文件大小，并且打印进度条，如果文件大小一样了，就结束线程
class MyThread(threading.Thread):
    def __init__(self, file_size, file_name):
        super(MyThread, self).__init__()
        # 进度条的长度
        self.bar_length = 50
        self.is_stop = False
        self.process_ = 0
        self.file_size = file_size
        if self.file_size.endswith('M'):
            self.file_size = int(float(self.file_size[:-1]) * 1024 * 1024)
        elif self.file_size.endswith('G'):
            self.file_size = int(float(self.file_size[:-1]) * 1024 * 1024 * 1024)
        self.file_name = file_name
        print('文件名：', self.file_name)
        self.daemon = True
        self.start()

    def get_process(self):
        yield self.process_

    def print_progress_bar(self, percent):
        # 进度条的长度
        bar_length = 50
        # 计算进度条的长度
        hashes = '#' * int(percent / 100 * bar_length)
        # 计算空格的长度
        spaces = ' ' * (bar_length - len(hashes))
        # 打印进度条
        print('\rPercent: [%s] %d%%' % (hashes + spaces, percent), end='')

    def run(self):
        print('开始下载文件：', self.file_name)
        while not self.is_stop:
            try:
                time.sleep(0.5)
                # 下载的文件为临时文件，会在原来的文件名后面加上.随机字符串，所以需要获取临时文件的大小
                # 获取文件名
                file_path = os.path.dirname(self.file_name)
                file_name = os.path.basename(self.file_name)
                # 获取临时文件名
                file_name_list = os.listdir(file_path)
                # print(file_name_list)
                file_name = [i for i in file_name_list if i.startswith(file_name)]
                file_name = file_name[0] if file_name else None
                # 如果为空，则continue
                if not file_name:
                    continue
                # 获取文件名
                # 获取文件大小
                file_size = os.path.getsize(file_name)
                # 计算百分比
                percent = file_size / self.file_size * 100
                # 四舍五入取整
                percent = 100 if int(round(percent)) > 97 else int(round(percent))
                self.process_ = percent
                # self.print_progress_bar(self.process_)
                # 如果文件大小一样了，就结束线程
                if file_size == self.file_size:
                    break
            except Exception as e:
                print(e)

    def stop(self):
        self.process_ = 100
        # self.print_progress_bar(self.process_)
        time.sleep(1)
        self.is_stop = True


def print_process(tmp_thread, callback=None):
    while not tmp_thread.is_stop:
        if callback:
            file_name = tmp_thread.file_name
            # file_name是一个绝对路径，需要截取出来，并且判断平台，如果是windows则需要把\替换成/
            if os.name == 'nt':
                file_name = file_name.split('\\')[-1]
            else:
                file_name = file_name.split('/')[-1]
            callback(tmp_thread.process_, file_name)
        time.sleep(1)


# 写一个装饰器，用来装饰下载文件的函数，并对函数的返回值进行迭代出来
def download_file(func):
    def wrapper(*args, **kwargs):
        # 获取传参中的callback
        callback = kwargs.get('callback')
        # 获取函数的返回值
        res = func(*args, **kwargs)
        # 迭代出来
        for thread_tmp in res:
            # 如果返回的是一个线程，则开辟一个线程打印日志
            if isinstance(thread_tmp, MyThread):
                threading.Thread(target=print_process, args=(thread_tmp, callback), daemon=True).start()
            else:
                return thread_tmp

    return wrapper


def ssh_is_file_exist(remote_path, ip, user_name, password, port=22):
    ssh = pexpect.spawn('ssh -p %s %s@%s ls %s' % (port, user_name, ip, remote_path))
    try:
        # i = ssh.expect([password_key, pexpect.EOF, pexpect.TIMEOUT], timeout=60)
        # 添加yes/no的判断
        i = ssh.expect([is_input_yes, password_key, pexpect.EOF, pexpect.TIMEOUT], timeout=60)
        if i == 0:
            ssh.sendline('yes')
            ssh.expect(password_key)
            ssh.sendline(password)
        elif i == 1:
            ssh.sendline(password)
        ssh.expect(pexpect.EOF)
        file_list_str = ssh.before
        if isinstance(file_list_str, bytes):
            file_list_str = file_list_str.decode('utf-8')
        if 'No such file or directory' in file_list_str:
            return False
        return True
    except pexpect.EOF:
        print('EOF')
        ssh.close()
        return False


# 获取列表下所有文件以及文件夹，并且获取这些文件的大小以及更新时间, 自动输入密码
def ssh_get_file_list(remote_path, ip, user_name, password, port=22):
    # ssh = pexpect.spawn('ssh %s@%s ls -l %s' % (user_name, ip, remote_path))
    ssh = pexpect.spawn('ssh -p %s %s@%s ls -l %s' % (port, user_name, ip, remote_path))
    try:
        i = ssh.expect([is_input_yes, password_key, pexpect.EOF, pexpect.TIMEOUT], timeout=60)
        if i == 0:
            ssh.sendline('yes')
            ssh.expect(password_key)
            ssh.sendline(password)
        elif i == 1:
            ssh.sendline(password)
        ssh.expect(pexpect.EOF)
        file_list_str = ssh.before
        # 如果file_list_str是bytes类型，则转换成str类型
        if isinstance(file_list_str, bytes):
            file_list_str = file_list_str.decode('utf-8')
        if 'No such file or directory' in file_list_str:
            return None
        file_list = file_list_str.split('\r\n')
        for i in file_list:
            if i.count('drwxr') > 1:
                new_i =[f'drwxr{o}' for o in i.split('drwxr')]
                file_list = new_i + file_list
        # print(f'file_list11: {file_list}')
        file_list = file_list[1:-1] if os.name != 'nt' else file_list[1:]
        # 对列表进行处理成一个字典的列表，键分别为权限，文件个数，所有者，所属组，大小，更新时间，文件名
        file_list_dict = []
        for f in file_list:
            file_dict = {}
            f = f.split()
            if len(f) != 9:
                continue
            file_dict['permissions'] = f[0]
            file_dict['number'] = f[1]
            file_dict['owner'] = f[2]
            file_dict['group'] = f[3]
            file_dict['size'] = f[4]
            file_dict['time'] = f[5] + ' ' + f[6] + ' ' + f[7]
            file_dict['name'] = f[8]
            file_list_dict.append(file_dict)
        return file_list_dict
    except pexpect.EOF:
        print('EOF')
        ssh.close()
        return None
    except pexpect.TIMEOUT:
        print('TIMEOUT')
        ssh.close()
        return None

@download_file
def ssh_download_file(remote_path, local_path, ip, user_name, password, port=22, callback=None):
    # 使用scp下载文件，同时使用-c参数，使用压缩传输，提高传输速度
    cmd = 'scp -P %s %s@%s:%s %s' % (port, user_name, ip, remote_path, local_path)
    # cmd = 'scp -c -P %s %s@%s:%s %s' % (port, user_name, ip, remote_path, local_path)
    print(cmd)
    ssh = pexpect.spawn(cmd)
    # 不设置超时时间，因为下载文件的时间不确定
    try:
        get_file_size_index = 0
        file_size = ssh_get_file_size(remote_path, ip, user_name, password, port)
        # 获取文件大小
        while True:
            if file_size:
                break
            get_file_size_index += 1
            if get_file_size_index > 3:
                print('获取文件大小失败')
                file_size = '0M'
        # 开启进度条打印线程
        file_name = remote_path.split('/')[-1]
        # 如果local_path没有以/结尾，则添加/
        if local_path[-1] != '/':
            local_path += '/'
        local_file_path = local_path + file_name
        # 如果本地文件已经存在，则删除
        if os.path.exists(local_file_path):
            # 删除localfile的只读属性
            # 如果是windows系统
            if os.name == 'nt':
                os.system(f"attrib -r {local_file_path}")
            else:
                # 给文件添加可写权限
                os.chmod(local_file_path, stat.S_IWRITE)
            os.remove(local_file_path)
            time.sleep(1)
        i = ssh.expect([is_input_yes, password_key, pexpect.EOF, pexpect.TIMEOUT])
        if i == 0:
            ssh.sendline('yes')
            ssh.expect(password_key)
            ssh.sendline(password)
        elif i == 1:
            ssh.sendline(password)
        process_thread = MyThread(file_size=file_size, file_name=local_file_path)
        yield process_thread
        ssh.expect(pexpect.EOF, timeout=6000)
        process_thread.stop()
        # 如果下载失败
        before_str = ssh.before
        if isinstance(before_str, bytes):
            before_str = before_str.decode('utf-8')
        if 'rsync error' in before_str:
            print('rsync error')
            ssh.close()
            yield False
        else:
            print('rsync success')
            ssh.close()
        yield True
    except pexpect.EOF:
        print('EOF')
        ssh.close()
        yield False

    
        

# @download_file
# def ssh_download_file(remote_path, local_path, ip, user_name, password, port=22, callback=None):
#     # cmd = 'scp %s@%s:%s %s' % (user_name, ip, remote_path, local_path)
#     cmd = 'scp -P %s %s@%s:%s %s' % (port, user_name, ip, remote_path, local_path)
#     print(cmd)
#     ssh = pexpect.spawn(cmd)
#     # 不设置超时时间，因为下载文件的时间不确定
#     try:
#         # 获取文件大小
#         file_size = ssh_get_file_size(remote_path, ip, user_name, password, port)
#         # 开启进度条打印线程
#         file_name = remote_path.split('/')[-1]
#         # 如果local_path没有以/结尾，则添加/
#         if local_path[-1] != '/':
#             local_path += '/'
#         local_file_path = local_path + file_name
#         # 如果本地文件已经存在，则删除
#         if os.path.exists(local_file_path):
#             os.remove(local_file_path)
#             time.sleep(1)
#         i = ssh.expect([is_input_yes, password_key, pexpect.EOF, pexpect.TIMEOUT])
#         if i == 0:
#             ssh.sendline('yes')
#             ssh.expect(password_key)
#             ssh.sendline(password)
#         elif i == 1:
#             ssh.sendline(password)
#         process_thread = MyThread(file_size=file_size, file_name=local_file_path)
#         yield process_thread
#         ssh.expect(pexpect.EOF, timeout=6000)
#         process_thread.stop()
#         # 如果下载失败
#         before_str = ssh.before
#         if isinstance(before_str, bytes):
#             before_str = before_str.decode('utf-8')
#         # print(f'ssh before: {before_str}')
#         if 'scp:' in before_str:
#             yield False
#         yield True
#     except pexpect.EOF:
#         print('EOF')
#         ssh.close()
#         yield False


def ssh_upload_file(local_path, remote_path, ip, user_name, password, port=22):
    # cmd = 'scp %s %s@%s:%s' % (local_path, user_name, ip, remote_path)
    # 先判断文件是否存在，如果存在，则把原来文件名重命名成_bak
    if ssh_is_file_exist(remote_path + '/' + local_path, ip, user_name, password, port):
        ssh_rename_file(remote_path + '/' + local_path, remote_path + '/' + local_path + '_bak', ip, user_name,
                        password, port)
    cmd = 'scp -P %s %s %s@%s:%s' % (port, local_path, user_name, ip, remote_path)
    ssh = pexpect.spawn(cmd)
    # 不设置超时时间，因为下载文件的时间不确定
    try:
        i = ssh.expect([is_input_yes, password_key, pexpect.EOF, pexpect.TIMEOUT], timeout=60)
        if i == 0:
            ssh.sendline('yes')
            ssh.expect(password_key)
            ssh.sendline(password)
        elif i == 1:
            ssh.sendline(password)
        ssh.expect(pexpect.EOF)
        before_str = ssh.before
        if isinstance(before_str, bytes):
            before_str = before_str.decode('utf-8')
        print(f'ssh before: {before_str}')
        # 如果100%都没有出现，则说明上传失败
        if '100%' not in before_str:
            ssh.close()
            return False
        ssh.close()
        return True
    except pexpect.EOF:
        print('EOF')
        ssh.close()
        return False


def ssh_rename_file(old_path, new_path, ip, user_name, password, port=22):
    # 先判断文件是否存在
    if not ssh_get_file_list(old_path, ip, user_name, password, port):
        return False
    # cmd = 'ssh %s@%s mv %s %s' % (user_name, ip, old_path, new_path)
    cmd = 'ssh -p %s %s@%s mv %s %s' % (port, user_name, ip, old_path, new_path)
    ssh = pexpect.spawn(cmd)
    try:
        i = ssh.expect([is_input_yes, password_key, pexpect.EOF, pexpect.TIMEOUT], timeout=60)
        if i == 0:
            ssh.sendline('yes')
            ssh.expect(password_key)
            ssh.sendline(password)
        elif i == 1:
            ssh.sendline(password)
        ssh_before = ssh.expect(pexpect.EOF)
        print(f'ssh before: {ssh_before}')
        return True
    except pexpect.EOF:
        print('EOF')
        ssh.close()
        return False
    except pexpect.TIMEOUT:
        print('TIMEOUT')
        ssh.close()
        return False


def ssh_delete_file(path, ip, user_name, password, port=22):
    # 先判断文件是否存在
    if not ssh_get_file_list(path, ip, user_name, password, port):
        return False
    # cmd = 'ssh %s@%s rm -rf %s' % (user_name, ip, path)
    cmd = 'ssh -p %s %s@%s rm -rf %s' % (port, user_name, ip, path)
    # print(f'delete file cmd: {cmd}')
    ssh = pexpect.spawn(cmd)
    try:
        i = ssh.expect([is_input_yes, password_key, pexpect.EOF, pexpect.TIMEOUT], timeout=60)
        if i == 0:
            ssh.sendline('yes')
            ssh.expect(password_key)
            ssh.sendline(password)
        elif i == 1:
            ssh.sendline(password)
        ssh_before = ssh.expect(pexpect.EOF)
        print(f'ssh before: {ssh_before}')
        return True
    except pexpect.EOF:
        print('EOF')
        ssh.close()
        return False
    except pexpect.TIMEOUT:
        print('TIMEOUT')
        ssh.close()
        return False

def ssh_delete_dir(path, ip, user_name, password, port=22):
    # cmd = 'ssh %s@%s rm -rf %s' % (user_name, ip, path)
    cmd = 'ssh -p %s %s@%s rm -rf %s' % (port, user_name, ip, path)
    # print(f'delete dir cmd: {cmd}')
    ssh = pexpect.spawn(cmd)
    try:
        i = ssh.expect([is_input_yes, password_key, pexpect.EOF, pexpect.TIMEOUT], timeout=60)
        if i == 0:
            ssh.sendline('yes')
            ssh.expect(password_key)
            ssh.sendline(password)
        elif i == 1:
            ssh.sendline(password)
        ssh.expect(pexpect.EOF)
        return True
    except pexpect.EOF:
        print('EOF')
        ssh.close()
        return False
    except pexpect.TIMEOUT:
        print('TIMEOUT')
        ssh.close()
        return False


def ssh_create_file(path, ip, user_name, password, port=22):
    # 先判断path是否存在
    if ssh_get_file_list(path, ip, user_name, password, port):
        return False
    # cmd = 'ssh %s@%s touch %s' % (user_name, ip, path)
    cmd = 'ssh -p %s %s@%s touch %s' % (port, user_name, ip, path)
    ssh = pexpect.spawn(cmd)
    try:
        i = ssh.expect([is_input_yes, password_key, pexpect.EOF, pexpect.TIMEOUT], timeout=60)
        if i == 0:
            ssh.sendline('yes')
            ssh.expect(password_key)
            ssh.sendline(password)
        elif i == 1:
            ssh.sendline(password)
        ssh.expect(pexpect.EOF)
        return True
    except pexpect.EOF:
        print('EOF')
        ssh.close()
        return False
    except pexpect.TIMEOUT:
        print('TIMEOUT')
        ssh.close()
        return False


def ssh_create_dir(path, ip, user_name, password, port=22):
    # 先判断path是否存在
    if ssh_get_file_list(path, ip, user_name, password, port):
        return False
    # cmd = 'ssh %s@%s mkdir %s' % (user_name, ip, path)
    cmd = 'ssh -p %s %s@%s mkdir %s' % (port, user_name, ip, path)
    print(f'create dir cmd: {cmd}')
    ssh = pexpect.spawn(cmd)
    try:
        i = ssh.expect([is_input_yes, password_key, pexpect.EOF, pexpect.TIMEOUT], timeout=60)
        if i == 0:
            ssh.sendline('yes')
            ssh.expect(password_key)
            ssh.sendline(password)
        elif i == 1:
            ssh.sendline(password)
        # 执行完命令后，会有一个EOF，需要expect一下，并打印出来
        ssh_before = ssh.expect(pexpect.EOF)
        print(f'ssh before: {ssh_before}')
        ssh.close()
        return True
    except pexpect.EOF:
        print('EOF')
        ssh.close()
        return False
    except pexpect.TIMEOUT:
        print('TIMEOUT')
        ssh.close()
        return False


def ssh_get_file_size(path, ip, user_name, password, port=22):
    # cmd = 'ssh %s@%s du -sh %s' % (user_name, ip, path)
    cmd = 'ssh -p %s %s@%s du -sh %s' % (port, user_name, ip, path)
    print(f'get file size cmd: {cmd}')
    ssh = pexpect.spawn(cmd)
    try:
        i = ssh.expect([is_input_yes, password_key, pexpect.EOF, pexpect.TIMEOUT], timeout=60)
        print(f'get file size i: {i}')
        if i == 0:
            ssh.sendline('yes')
            ssh.expect(password_key)
            ssh.sendline(password)
        elif i == 1:
            ssh.sendline(password)
        ssh.expect(pexpect.EOF)
        before_str = ssh.before
        # 判断before_str是为bytes类型还是str类型
        if isinstance(before_str, bytes):
            before_str = before_str.decode('utf-8')
        file_size = before_str.split('\r\n')[1].split()[0]
        return file_size
    except pexpect.EOF:
        print('EOF')
        ssh.close()
        return None
    except pexpect.TIMEOUT:
        print('TIMEOUT')
        ssh.close()
        return None
    
# 写一个异步下载文件的方法，使用回调函数能够实时获取下载进度以及下载速度
async def ssh_download_file_async(path, local_path, ip, user_name, password, port=22, callback=None):
    # 先判断path是否存在
    if not ssh_get_file_list(path, ip, user_name, password, port):
        return False
    # cmd = 'scp %s@%s:%s %s' % (user_name, ip, path, local_path)
    cmd = 'scp -P %s %s@%s:%s %s' % (port, user_name, ip, path, local_path)
    print(f'download file cmd: {cmd}')
    ssh = pexpect.spawn(cmd)
    try:
        i = ssh.expect([is_input_yes, password_key, pexpect.EOF, pexpect.TIMEOUT], timeout=60)
        print(f'download file i: {i}')
        if i == 0:
            ssh.sendline('yes')
            ssh.expect(password_key)
            ssh.sendline(password)
        elif i == 1:
            ssh.sendline(password)
        ssh.expect(pexpect.EOF)
        return True
    except pexpect.EOF:
        print('EOF')
        ssh.close()
        return False
    except pexpect.TIMEOUT:
        print('TIMEOUT')
        ssh.close()
        return False
        


def process_callback(p, f):
    print(p, f)


if __name__ == '__main__':
    ip = '39.101.69.111'
    user_name = 'root'
    password = 'Dxr85578595'
    # res = ssh_download_file('/root/test.zip', './', ip, user_name, password, callback=process_callback)
    # print(ssh_is_file_exist('/root/test1.zip', ip, user_name, password))
    # print(ssh_get_file_list('/root1', ip, user_name, password))
    # print(ssh_upload_file('./error.log', '/root', ip, user_name, password))
    # print(ssh_rename_file('/root/error1.log', '/root/error.log', ip, user_name, password))
    # print(ssh_delete_file('/root/error.log', ip, user_name, password))
    # print(ssh_create_dir('/root/test', ip, user_name, password))
    print(ssh_delete_dir('/root/test', ip, user_name, password))


"""
hikvision.api
~~~~~~~~~~~~~~~~~~~~

Provides methods for interacting with hikvision

Copyright (c) 2015 Finbarr Brady <https://github.com/fbradyirl>
Licensed under the MIT license.
"""

import logging
from xml.etree import ElementTree
import re

import requests
from requests.exceptions import ConnectionError as ReConnError
from requests.auth import HTTPBasicAuth, HTTPDigestAuth

from .error import HikvisionError, MissingParamError
from .constants import DEFAULT_PORT, DEFAULT_HEADERS, XML_ENCODING
from .constants import DEFAULT_SENS_LEVEL

_LOGGING = logging.getLogger(__name__)

# pylint: disable=too-many-arguments
# pylint: disable=too-many-instance-attributes


def build_url_base(host, port, is_https):
    """
    Make base of url based on config
    """
    base = "http"
    if is_https:
        base += 's'

    base += "://"
    base += host

    if port:
        base += ":"
        base += str(port)

    return base


def log_response_errors(response):
    """
    Logs problems in a response
    """

    _LOGGING.error("status_code %s", response.status_code)


def enable_logging():
    """ Setup the logging for home assistant. """
    logging.basicConfig(level=logging.INFO)


def remove_namespace(response):
    """ Removes namespace element from xml"""
    return re.sub(' xmlns="[^"]+"', '', response)


def tree_no_ns_from_string(response):
    """ Removes namespace element from response"""
    text = remove_namespace(response)
    return ElementTree.fromstring(text)


class CreateDevice:

    """
    Creates a new camera api device
    """

    def __init__(self, host=None, port=DEFAULT_PORT,
                 username=None, password=None, is_https=False,
                 sensitivity_level=DEFAULT_SENS_LEVEL,
                 digest_auth=True, strict_isapi=True):
        enable_logging()
        # _LOGGING.info("Initialising new hikvision camera client")

        if not host:
            # _LOGGING.error('Missing hikvision host!')
            raise MissingParamError('Connection to hikvision failed.', None)

        if not digest_auth and not is_https:
            # _LOGGING.warning("%s: HTTP Basic Auth without SSL is insecure",
            #                  host)
            pass

        self._username = username
        self._host = host
        self._password = password
        self._sensitivity_level = sensitivity_level
        self._digest_auth = digest_auth
        self._strict_isapi = strict_isapi
        self._auth_fn = HTTPDigestAuth if self._digest_auth else HTTPBasicAuth
        self.xml_motion_detection_off = None
        self.xml_motion_detection_on = None

        # Now build base url
        self._base = build_url_base(host, port, is_https)

        # need to support different channel
        if self._strict_isapi:
            self.motion_url = (
                '%s/ISAPI/System/Video/Inputs/channels/1/motionDetection' %
                self._base)
            self.deviceinfo_url = '%s/ISAPI/System/deviceInfo' % self._base
            self.ptzctrl_url = '%s/ISAPI/PTZCtrl/channels/1/continuous' % self._base
            self.ptzpreset_url = '%s/ISAPI/PTZCtrl/channels/1/presets' % self._base
            self.pic_url = '%s/ISAPI/Streaming/channels/1/picture' % self._base
            self.rtsp_url = '%s/ISAPI/Streaming/channels/101' % self._base
            self.ptz_senior_url = '%s/ISAPI/PTZCtrl/channels/1/absoluteExt' % self._base
            self.set_preset_url = '%s/ISAPI/PTZCtrl/channels/1/presets' % self._base
            self.ptzfocus_url = '%s/ISAPI/System/Video/inputs/channels/1/focus' % self._base
            # self.ir_url = '%s/ISAPI/Thermal/channels/2/thermometry/pixelToPixelParam/capabilities' % self._base
            self.ir_url = '%s/ISAPI/Thermal/channels/2/thermometry/jpegPicWithAppendData?format=json' % self._base
            self.capabilities_url = '%s/ISAPI/PTZCtrl/channels/1/absoluteEx/capabilities' % self._base
            self.ptz_absoluteEx_url = '%s/ISAPI/PTZCtrl/channels/1/absoluteEx' % self._base
            self.onepushfocus_url = '%s/ISAPI/PTZCtrl/channels/1/onepushfoucs/start' % self._base
            self.absoluteStatus_url = '%s/ISAPI/PTZCtrl/channels/1/status' % self._base
            self.imageFocus_url = '%s/ISAPI/Image/channels/1/focusConfiguration' % self._base
            self.ptz_capabilities_url = '%s/ISAPI/PTZCtrl/capabilities' % self._base
            self.ptz_autopan_url = '%s/ISAPI/PTZCtrl/channels/1/autopan' % self._base
            self.streamParam_url = '%s/ISAPI/Thermal/channels/2/streamParam' % self._base
            self.auxcontrol_url = '%s/ISAPI/PTZCtrl/channels/1/auxcontrols/1' % self._base

        else:
            self.motion_url = '%s/MotionDetection/1' % self._base
            self.deviceinfo_url = '%s/System/deviceInfo' % self._base
            self.ptzctrl_url = '%s/PTZCtrl/capabilities' % self._base
            self.ptzpreset_url = '%s/PTZCtrl/channels/1/presets' % self._base
            self.pic_url = '%s/System/Video/channels/1/picture' % self._base
            self.rtsp_url = '%s/Streaming/channels/1' % self._base
            self.ptz_senior_url = '%s/PTZCtrl/channels/1/absoluteExt' % self._base
            self.ir_url = '%s/Thermal/channels/2/thermometry/pixelToPixelParam/capabilities?format=json' % self._base
#            self._xml_namespace = "{http://www.hikvision.com/ver10/XMLSchema}"
        self._xml_namespace = ""

    
    def get_ptz_capabilities(self, element_to_query=None):
        response = requests.get(
            self.ptz_capabilities_url,
            auth=self._auth_fn(self._username, self._password),
            headers={'Content-Type': 'application/json'}
            )
        if response.status_code != 200:
            return None

        if element_to_query is None:
            # print(response.text)
            # 返回的是json格式的数据，返回的是一个字典
            return response.text
        try:
            tree = tree_no_ns_from_string(response.text)

            element_to_query = './/%s%s' % (
                self._xml_namespace, element_to_query)
            result = tree.findall(element_to_query)
            if len(result) > 0:
                

                return result[0].text.strip()
            

        except AttributeError as attib_err:
            
            return None
        return None
    

    def get_absolutestatus(self, element_to_query=None):
        response = requests.get(
            self.ptz_absoluteEx_url,
            auth=self._auth_fn(self._username, self._password),
            headers={'Content-Type': 'application/json'}
            )
        if response.status_code != 200:
            return None

        if element_to_query is None:
            # print(response.text)
            # 返回的是json格式的数据，返回的是一个字典
            return response.text
        try:
            tree = tree_no_ns_from_string(response.text)

            element_to_query = './/%s%s' % (
                self._xml_namespace, element_to_query)
            result = tree.findall(element_to_query)
            if len(result) > 0:
                

                return result[0].text.strip()
            

        except AttributeError as attib_err:
            
            return None
        return None
    
    def set_ptzabsoluteEx(self,elevation,azimuth,absoluteZoom ,focus,focalLen,horizontalSpeed=10.00,verticalSpeed=10.00,zoomType='focalLen',element_to_query=None):
        data_xml = '<?xml version="1.0" encoding="UTF-8"?>' +\
                    '<PTZAbsoluteEx version="2.0" xmlns="http://www.isapi.org/ver20/XMLSchema">' +\
                    '<elevation>'+ str(elevation) +'</elevation>'+\
                    '<azimuth>'+ str(azimuth) +'</azimuth>'+\
                    '<absoluteZoom>'+ str(absoluteZoom) +'</absoluteZoom>'+\
                    '<focus>'+ str(focus) +'</focus>'+\
                    '<focalLen>'+ str(focalLen) +'</focalLen>'+\
                    '<horizontalSpeed>'+ str(horizontalSpeed) +'</horizontalSpeed>'+\
                    '<verticalSpeed>'+ str(verticalSpeed) +'</verticalSpeed>'+\
                    '<zoomType>'+ str(zoomType) +'</zoomType>'+\
                    '</PTZAbsoluteEx>'
        # 转换为UTF-8编码
        data_xml = data_xml.encode('utf-8')
        # print(data_xml)
        response = requests.put(
            self.ptz_absoluteEx_url,
            auth=self._auth_fn(self._username, self._password),
            data=data_xml,
            )
        if response.status_code != 200:
            print('http code:',response.status_code)
            print('response:',response.text)
            # log_response_errors(response)
            return None

        if element_to_query is None:
            # print(response.status_code)
            # print(response.text)
            return response.text
        try:
            tree = tree_no_ns_from_string(response.text)

            element_to_query = './/%s%s' % (
                self._xml_namespace, element_to_query)
            result = tree.findall(element_to_query)
            if len(result) > 0:
                return result[0].text.strip()
        except AttributeError as attib_err:
            return None
        return None
    
    def set_onepushfocus(self,element_to_query=None):
        response = requests.put(
            self.onepushfocus_url,
            auth=self._auth_fn(self._username, self._password),
            headers={'Content-Type': 'application/json'}
            )
        if response.status_code != 200:
            return response.text

        if element_to_query is None:
            # 返回的是json格式的数据，返回的是一个字典
            return response.text
        try:
            tree = tree_no_ns_from_string(response.text)

            element_to_query = './/%s%s' % (
                self._xml_namespace, element_to_query)
            result = tree.findall(element_to_query)
            if len(result) > 0:
                

                return result[0].text.strip()
            

        except AttributeError as attib_err:
            
            return None
        return None
    
    def set_auopan(self,autoPan=0,element_to_query=None):
        autoPan = 60 * autoPan
        data_xml = '<?xml version="1.0" encoding="UTF-8"?><autoPanData><autoPan>' + str(autoPan) + '</autoPan></autoPanData> '
        # 转换为UTF-8编码
        data_xml = data_xml.encode('utf-8')
        # print(data_xml)
        response = requests.put(
            self.ptz_autopan_url,
            auth=self._auth_fn(self._username, self._password),
            data=data_xml,
            )
        if response.status_code != 200:
            print('http code:',response.status_code)
            print('response:',response.text)
            # log_response_errors(response)
            return None

        if element_to_query is None:
            # print(response.text)
            return response.text
        try:
            tree = tree_no_ns_from_string(response.text)

            element_to_query = './/%s%s' % (
                self._xml_namespace, element_to_query)
            result = tree.findall(element_to_query)
            if len(result) > 0:
                return result[0].text.strip()
        except AttributeError as attib_err:
            return None
        return None
    
    def ir_test(self, element_to_query=None):
        response = requests.get(
            self.ir_url,
            auth=self._auth_fn(self._username, self._password),
            headers={'Content-Type': 'application/json'}
            )
        if response.status_code != 200:
            log_response_errors(response)
            return None

        if element_to_query is None:
            # 返回的是json格式的数据，返回的是一个字典
            return response.json()
        try:
            tree = tree_no_ns_from_string(response.text)

            element_to_query = './/%s%s' % (
                self._xml_namespace, element_to_query)
            result = tree.findall(element_to_query)
            if len(result) > 0:
                

                return result[0].text.strip()
            

        except AttributeError as attib_err:
            
            return None
        return None
    
    def get_pic(self, element_to_query=None):
        response = requests.get(
            self.pic_url,
            auth=self._auth_fn(self._username, self._password),
            )
        if response.status_code != 200:
            return None

        if element_to_query is None:
            return response.text
        try:
            tree = tree_no_ns_from_string(response.text)

            element_to_query = './/%s%s' % (
                self._xml_namespace, element_to_query)
            result = tree.findall(element_to_query)
            if len(result) > 0:
                return result[0].text.strip()
            

        except AttributeError as attib_err:
            
            return None
        return None
    
    def set_preset(self,  preset_number: int, element_to_query=None):
        set_p_url = self.set_preset_url + '/' + str(preset_number)
        data_xml = '<?xml version: "1.0" encoding="UTF-8"?>'+\
        '<PTZPreset xmlns="http://www.isapi.org/ver20/XMLSchema" version="2.0">'+\
            '<id>'+str(preset_number)+'</id>'+\
            '<presetName>预置点 '+str(preset_number)+'</presetName>'+\
        '</PTZPreset>'
        # 转换为UTF-8编码
        data_xml = data_xml.encode('utf-8')
        response = requests.put(
            set_p_url,
            auth=self._auth_fn(self._username, self._password),
            data=data_xml,
            )
        if response.status_code != 200:
            print('something wrong',response)
            # log_response_errors(response)
            return None

        if element_to_query is None:
            return response.text
        try:
            tree = tree_no_ns_from_string(response.text)

            element_to_query = './/%s%s' % (
                self._xml_namespace, element_to_query)
            result = tree.findall(element_to_query)
            if len(result) > 0:
                return result[0].text.strip()
        except AttributeError as attib_err:
            return None
        return None
    
    def goto_preset(self,  preset_number: int, element_to_query=None):
        ptz_preset_url = self.ptzpreset_url + '/' + str(preset_number) + '/goto'
        response = requests.put(
            ptz_preset_url,
            auth=self._auth_fn(self._username, self._password),
            )
        if response.status_code != 200:
            return None

        if element_to_query is None:
            return response.text
        try:
            tree = tree_no_ns_from_string(response.text)

            element_to_query = './/%s%s' % (
                self._xml_namespace, element_to_query)
            result = tree.findall(element_to_query)
            if len(result) > 0:
                return result[0].text.strip()

        except AttributeError as attib_err:
            return None
        return None
    
    def delete_preset(self,preset_number: int, element_to_query=None):
        ptz_preset_url = self.ptzpreset_url + \
            '/' + str(preset_number)
        response = requests.delete(
            ptz_preset_url,
            auth=self._auth_fn(self._username, self._password),
        )
        if response.status_code != 200:
            return None

        if element_to_query is None:
            return response.text
        try:
            tree = tree_no_ns_from_string(response.text)

            element_to_query = './/%s%s' % (
                self._xml_namespace, element_to_query)
            result = tree.findall(element_to_query)
            if len(result) > 0:
                return result[0].text.strip()

        except AttributeError as attib_err:

            return None
        return None
    
    # 聚焦控制 0-停止 1-聚焦+ -1-聚焦-
    def ptz_focus(self,add_sub = 0,element_to_query=None):
        focus = 60 * add_sub
        data_xml = '<?xml version: "1.0" encoding="UTF-8"?><FocusData><focus>'+ str(focus) +'</focus></FocusData>'
        response = requests.put(
            self.ptzfocus_url,
            auth=self._auth_fn(self._username, self._password),
            data=data_xml,
            )
        if response.status_code != 200:
            print(response)
            return response.text

        if element_to_query is None:
            return response.text
        try:
            tree = tree_no_ns_from_string(response.text)

            element_to_query = './/%s%s' % (
                self._xml_namespace, element_to_query)
            result = tree.findall(element_to_query)
            if len(result) > 0:
                return result[0].text.strip()

        except AttributeError as attib_err:
            return None
        return None
    
    def ptz_senior(self, element_to_query=None):
        data_xml = '<?xml version="1.0" encoding="UTF-8"?>'+\
        '<PTZAbsoluteEx xmlns="http://www.isapi.org/ver20/XMLSchema" version="2.0">'+\
            '<elevation>0.000</elevation>'+\
            '<azimuth>0.000</azimuth>'+\
            '<absoluteZoom>0</absoluteZoom>'+\
            '<focus>1</focus>'+\
            '<focalLen>1</focalLen>'+\
            '<horizontalSpeed>0.00</horizontalSpeed>'+\
            '<verticalSpeed>0.00</verticalSpeed>'+\
            '<zoomType>%s</zoomType>'+\
        '</PTZAbsoluteEx>'
        response = requests.get(
            self.ptz_senior_url,
            auth=self._auth_fn(self._username, self._password),
            data=data_xml,
            )
        if response.status_code != 200:
            log_response_errors(response)
            return None

        if element_to_query is None:
            return response.text
        try:
            tree = tree_no_ns_from_string(response.text)

            element_to_query = './/%s%s' % (
                self._xml_namespace, element_to_query)
            result = tree.findall(element_to_query)
            if len(result) > 0:
                return result[0].text.strip()

        except AttributeError as attib_err:
            return None
        return None
    
    # 设置图像参数,聚焦模式,聚焦限制 focusStyle: SEMIAUTOMATIC 半自动, MANUAL 手动, AUTO 自动，focusLimited: 聚焦限制 int, [10#10厘米,50#50厘米,100#1米,150#1.5米,300#3米,600#6米,1000#10米,2000#20米,5000#50米,20000#200米,50000#500米,65535#655.35米] 
    def set_image_capabilities(self,focusStyle='SEMIAUTOMATIC',focusLimited=10, element_to_query=None):
        data_xml = '<?xml version: "1.0" encoding="UTF-8"?>' +\
        '<FocusConfiguration>' +\
        '<focusStyle>'+ str(focusStyle) +'</focusStyle>' +\
        '<focusLimited>'+ str(focusLimited) +'</focusLimited>'+\
        '</FocusConfiguration>' 
        
        response = requests.put(
            self.imageFocus_url,
            auth=self._auth_fn(self._username, self._password),
            data=data_xml,
            )
        if response.status_code != 200:
            print(response)
            return response.text

        if element_to_query is None:
            return response.text
        try:
            tree = tree_no_ns_from_string(response.text)

            element_to_query = './/%s%s' % (
                self._xml_namespace, element_to_query)
            result = tree.findall(element_to_query)
            if len(result) > 0:
                return result[0].text.strip()

        except AttributeError as attib_err:
            return None
        return None
    
    def set_ptz(self, element_to_query=None, up_down=0, left_right=0, zoom=0):
        pan =left_right * 15
        tilt = -up_down * 15
        zoom_ = zoom * 15
        # 最大值为正负100
        if pan > 100:
            pan = 100
        if pan < -100:
            pan = -100
        if tilt > 100:
            tilt = 100
        if tilt < -100:
            tilt = -100
        if zoom_ > 100:
            zoom_ = 100
        if zoom_ < -100:
            zoom_ = -100
        
        data_xml = '<?xml version: "1.0" encoding="UTF-8"?><PTZData><pan>%s</pan><tilt>%s</tilt><zoom>%s</zoom></PTZData>' % (pan, tilt, zoom_)
        response = requests.put(
            self.ptzctrl_url,
            auth=self._auth_fn(self._username, self._password),
            data=data_xml,
            )
        if response.status_code != 200:
            return None

        if element_to_query is None:
            return response.text
        try:
            tree = tree_no_ns_from_string(response.text)

            element_to_query = './/%s%s' % (
                self._xml_namespace, element_to_query)
            result = tree.findall(element_to_query)
            if len(result) > 0:
                return result[0].text.strip()
        except AttributeError as attib_err:
            return None
        return None
    
    def get_streamParam(self, element_to_query=None):
        response = requests.get(
            self.streamParam_url,
            auth=self._auth_fn(self._username, self._password),
            )
        if response.status_code != 200:
            return None

        if element_to_query is None:
            return response.text
        try:
            tree = tree_no_ns_from_string(response.text)

            element_to_query = './/%s%s' % (
                self._xml_namespace, element_to_query)
            result = tree.findall(element_to_query)
            if len(result) > 0:
                return result[0].text.strip()
            

        except AttributeError as attib_err:
            
            return None
        return None
    
    # 配置指定通道热成像码流参数 videoCodingType:thermal_raw_data#热成像裸数据,pixel-to-pixel_thermometry_data#全屏测温数据,real-time_raw_data#实时裸数据,spectrogramData#科学仪器谱图数据（包括检测物质中的光谱波长与能量值） 
    def set_streamParam(self, element_to_query=None, videoCodingType='pixel-to-pixel_thermometry_data'):
        data_xml = '<?xml version: "1.0" encoding="UTF-8"?><ThermalStreamParam xmlns="http://www.isapi.org/ver20/XMLSchema" version="2.0">' +\
            '<videoCodingType>'+ str(videoCodingType) +'</videoCodingType>' +\
            '</ThermalStreamParam>'
            
        print(data_xml)
            
        response = requests.put(
            self.streamParam_url,
            auth=self._auth_fn(self._username, self._password),
            data=data_xml,
            )
        if response.status_code != 200:
            return None

        if element_to_query is None:
            return response.text
        try:
            tree = tree_no_ns_from_string(response.text)

            element_to_query = './/%s%s' % (
                self._xml_namespace, element_to_query)
            result = tree.findall(element_to_query)
            if len(result) > 0:
                return result[0].text.strip()
        except AttributeError as attib_err:
            return None
        return None
    
    def get_version(self):
        """
        Returns the firmware version running on the camera
        """
        return self.get_about(element_to_query='firmwareVersion')

    def get_about(self, element_to_query=None):
        """
        Returns ElementTree containing the result of
        <host>/System/deviceInfo
        or if element_to_query is not None, the value of that element
        """

        response = requests.get(
            self.deviceinfo_url,
            auth=self._auth_fn(self._username, self._password))

        if response.status_code != 200:
            log_response_errors(response)
            return None

        if element_to_query is None:
            return response.text
        try:
            tree = tree_no_ns_from_string(response.text)

            element_to_query = './/%s%s' % (
                self._xml_namespace, element_to_query)
            result = tree.findall(element_to_query)
            if len(result) > 0:
                

                return result[0].text.strip()
            

        except AttributeError as attib_err:
            
            return None
        return None

    def is_motion_detection_enabled(self):
        """Get current state of Motion Detection.

        Returns False on error or if motion detection is off."""

        response = requests.get(self.motion_url, auth=self._auth_fn(
            self._username, self._password))


        if response.status_code != 200:
            return False

        try:

            tree = tree_no_ns_from_string(response.text)
            enabled_element = tree.findall(
                './/%senabled' % self._xml_namespace)
            sensitivity_level_element = tree.findall(
                './/%ssensitivityLevel' % self._xml_namespace)
            if len(enabled_element) == 0:
                return False
            if len(sensitivity_level_element) == 0:
                return False

            result = enabled_element[0].text.strip()
            
            if int(sensitivity_level_element[0].text) == 0:
                sensitivity_level_element[0].text = str(
                    self._sensitivity_level)

            if result == 'true':
                # Save this for future switch off
                self.xml_motion_detection_on = ElementTree.tostring(
                    tree, encoding=XML_ENCODING)
                enabled_element[0].text = 'false'
                self.xml_motion_detection_off = ElementTree.tostring(
                    tree, encoding=XML_ENCODING)
                return True
            # Save this for future switch on
            self.xml_motion_detection_off = ElementTree.tostring(
                tree, encoding=XML_ENCODING)
            enabled_element[0].text = 'true'
            self.xml_motion_detection_on = ElementTree.tostring(
                tree, encoding=XML_ENCODING)
            return False

        except AttributeError as attib_err:
            return False

    def enable_motion_detection(self):
        """ Enable Motion Detection """

        self.put_motion_detection_xml(self.xml_motion_detection_on)

    def disable_motion_detection(self):
        """ Disable Motion Detection """

        self.put_motion_detection_xml(self.xml_motion_detection_off)

    def put_motion_detection_xml(self, xml):
        """ Put request with xml Motion Detection """



        headers = DEFAULT_HEADERS
        headers['Content-Length'] = str(len(xml))
        headers['Host'] = self._host
        response = requests.put(self.motion_url, auth=self._auth_fn(
            self._username, self._password), data=xml, headers=headers)


        if response.status_code != 200:

            return

        try:
            tree = tree_no_ns_from_string(response.text)
            enabled_element = tree.findall(
                './/%sstatusString' % self._xml_namespace)
            if len(enabled_element) == 0:
                
                return

            if enabled_element[0].text.strip() == 'OK':
                pass

        except AttributeError as attrib_err:
            return
    
    # 打开雨刷
    def enable_wiper(self,status = 'on', element_to_query=None):
        """ Enable Wiper """
        data_xml = '<?xml version: "1.0" encoding="UTF-8"?><PTZAux><id>1</id><type>WIPER</type><status>'+ status +'</status></PTZAux>'
        response = requests.put(
            self.auxcontrol_url,
            auth=self._auth_fn(self._username, self._password),
            data=data_xml,
            )
        if response.status_code != 200:
            return None

        if element_to_query is None:
            return response.text
        try:
            tree = tree_no_ns_from_string(response.text)

            element_to_query = './/%s%s' % (
                self._xml_namespace, element_to_query)
            result = tree.findall(element_to_query)
            if len(result) > 0:
                return result[0].text.strip()
        except AttributeError as attib_err:
            return None
        return None
    
    # 补光灯控制
    def enable_light(self,status = 'on', element_to_query=None):
        """ Enable Light """
        data_xml = '<?xml version: "1.0" encoding="UTF-8"?><PTZAux><id>1</id><type>LIGHT</type><status>'+ status +'</status></PTZAux>'
        response = requests.put(
            self.auxcontrol_url,
            auth=self._auth_fn(self._username, self._password),
            data=data_xml,
            )
        if response.status_code != 200:
            return None

        if element_to_query is None:
            return response.text
        try:
            tree = tree_no_ns_from_string(response.text)

            element_to_query = './/%s%s' % (
                self._xml_namespace, element_to_query)
            result = tree.findall(element_to_query)
            if len(result) > 0:
                return result[0].text.strip()
        except AttributeError as attib_err:
            return None
        return None
    



import requests

# 使用requests库，发送get请求
def ssh_create_dir(path, ip, *args, **kwargs):
    # 通过requests库，发送POST请求, path是文件的绝对路径
    url = 'http://%s:6050/add_user' % ip
    print(f'url is {url}, path is {path}')
    data = {'user_name': path}
    try:
        r = requests.get(f'{url}{path.split()[-1]}', timeout=1)
        print(r.json())
        if r.json()['code'] == 200:
            return True
        else:
            return False
    except Exception as e:
        print(e)
        return False
    
# 使用requests库，发送post请求
def ssh_delete_dir(path, ip, *args, **kwargs):
    # 通过requests库，发送POST请求, path是文件的绝对路径
    url = 'http://%s:5000/delete_user' % ip
    data = {'user_name': path}
    try:
        r = requests.post(url, data=data, timeout=1)
        print(r.json())
        if r.json()['code'] == 200:
            return True
        else:
            return False
    except Exception as e:
        print(e)
        return False
    
def ssh_delete_file(path, ip, *args, **kwargs):
    # 通过requests库，发送POST请求, path是文件的绝对路径
    url = 'http://%s:5000/delete_image' % ip
    data = {'user_name': path}
    try:
        r = requests.post(url, data=data, timeout=1)
        print(r.json())
        if r.json()['code'] == 200:
            return True
        else:
            return False
    except Exception as e:
        print(e)
        return False
    
def ssh_upload_file(local_path, remote_path, ip, *args, **kwargs):
    # 通过requests库，发送POST请求, local_path是本地文件的绝对路径，remote_path是远程文件的绝对路径
    url = 'http://%s:5000/upload_image' % ip
    data = {'user_name': remote_path}
    files = {'file': open(local_path, 'rb')}
    try:
        r = requests.post(url, data=data, files=files, timeout=1)
        print(r.json())
        if r.json()['code'] == 200:
            return True
        else:
            return False
    except Exception as e:
        print(e)
        return False

def ssh_upload_face(local_path, remote_path, ip,, *args, **kwargs):
    # 通过requests库，发送POST请求, local_path是本地文件的绝对路径，remote_path是远程文件的绝对路径
    url = 'http://%s:81/upload_image' % ip
    data = {'user_name': remote_path}
    files = {'file': open(local_path, 'rb')}
    try:
        r = requests.post(url, data=data, files=files, timeout=1)
        print(r.json())
        if r.json()['code'] == 200:
            return True
        else:
            return False
    except Exception as e:
        print(e)
        return False
    
if __name__ == '__main__'
    print('删除文件夹')
    res = ssh_delete_dir('rootDXR_ProjectDXR_FaceRecfacedatabasetest', '10.10.11.93','root', '123456', port=22)
    print(res)
    # print('创建文件夹')
    # res = ssh_create_dir('rootDXR_ProjectDXR_FaceRecfacedatabasetest', '10.10.11.93','root', '123456', port=22)
    # print(res)
    # print('上传文件')
    # res = ssh_upload_file('dxr_ssh.py', 'rootDXR_ProjectDXR_FaceRecfacedatabasetest', '10.10.11.93','root', '123456', port=22)
    # print(res)
    # print('删除文件')
    # res = ssh_delete_file('rootface_tmpdxr_ssh.py', '10.10.11.93','root', '123456', port=22)
    # print(res)


import yaml

from . import gvalues
import os
import platform

ConfigFiles = []
KEY_ARR_TMP = dict()


def setModuleName(name):
    gvalues.module_name = name


def initConfig():
    global ConfigFiles
    if platform.system() == 'Windows':
        ConfigPath = 'D:\\Config'
    else:
        ConfigPath = os.path.expanduser('~') + '/Config'
    ConfigFiles = GetFileFromThisRootDir(ConfigPath, ['yaml'])
    print(ConfigFiles)
    gvalues.ConfigList = dict()
    for file in ConfigFiles:
        f = open(file['path'])
        gvalues.ConfigList[file['moduleName']] = yaml.load(f, Loader=yaml.FullLoader)
    print(gvalues.ConfigList)


# 获取指定路径下所有指定后缀的文件
# dir 指定路径
# ext 指定后缀，链表&不需要带点 或者不指定。例子：['xml', 'java']
def GetFileFromThisRootDir(dir, ext=None):
    allfiles = []
    needExtFilter = (ext is not None)
    for root, dirs, files in os.walk(dir):
        for filespath in files:
            filepath = os.path.join(root, filespath)
            fileName = filepath.split('/')[-1]
            moduleName = fileName.split('.')[0]
            extension = os.path.splitext(filepath)[1][1:]
            if needExtFilter and extension in ext:
                allfiles.append(
                    {
                        'path': filepath,
                        'fileName': fileName,
                        'moduleName': moduleName
                    }
                )
            elif not needExtFilter:
                allfiles.append(
                    {
                        'path': filepath,
                        'fileName': fileName,
                        'moduleName': moduleName
                    }
                )
    return allfiles


# 设置配置文件内容
def setConfig(moduleName, key, value):
    global ConfigFiles
    loader = gvalues.ConfigList[moduleName]

    key_arr = key.split('.')
    tmp_arr = []
    for key_str in key_arr:
        if len(tmp_arr) == 0:
            tmp_arr.append(loader[key_str])
        elif len(tmp_arr) == len(key_arr) - 1:
            tmp_arr[-1][key_str] = value
    for file in ConfigFiles:
        if file['moduleName'] == moduleName:
            with open(file['path'], 'w', encoding='utf-8') as w_f:
                # 覆盖原先的配置文件
                yaml.dump(loader, w_f)
            break


# 获取配置文件内容
def getConfig(moduleName, key):
    loader = gvalues.ConfigList[moduleName]
    key_arr = key.split('.')
    for key_str in key_arr:
        loader = loader[key_str]
    return loader


# 设置快捷键值对
def setKeyName(moduleName, old_key, new_key):
    global KEY_ARR_TMP
    KEY_ARR_TMP[new_key] = {
        'old_key': old_key,
        'module_name': moduleName
    }


# 使用快捷键获设置yaml内容
def setKey(new_key, value):
    global KEY_ARR_TMP
    if new_key in KEY_ARR_TMP:
        old_key = KEY_ARR_TMP[new_key]['old_key']
        moduleName = KEY_ARR_TMP[new_key]['module_name']
        setConfig(moduleName, old_key, value)


# 使用快捷键获取yaml内容
def getKey(new_key):
    global KEY_ARR_TMP
    if new_key in KEY_ARR_TMP:
        old_key = KEY_ARR_TMP[new_key]['old_key']
        moduleName = KEY_ARR_TMP[new_key]['module_name']
        return getConfig(moduleName, old_key)
    else:
        return ''





import json
import grpc
import cv2
import numpy as np
from . import Datas_pb2
from . import Datas_pb2_grpc
from threading import Thread
import time
            
class Dxr_grpc_client:
    def __init__(self, ip, port):
        self.ip = ip
        self.port = port
        self.channel = grpc.insecure_channel(self.ip + ':' + self.port)
        self.stub = Datas_pb2_grpc.MainServerStub(self.channel)
        # 启动线程，用来持续监听self.channel是否连接，如果断开则重新连接
        # self.thread = Thread(target=self._keep_connect, args=(), daemon=True)
        # self.thread.start()
        
    def _keep_connect(self):
        while True:
            time.sleep(1)
            # grpc_health.v1.Health.Check
            res = grpc.channel_ready_future(self.channel).result()
            if res is not None:
                print('channel is not ready, reconnecting...')
                self.channel = grpc.insecure_channel(self.ip + ':' + self.port)
                self.stub = Datas_pb2_grpc.MainServerStub(self.channel)
            else:
                pass
        
    def self_request(self, frame, json_data):
        ret, buf = cv2.imencode('.jpg', frame)
        if ret != 1:
            return
        yield Datas_pb2.Request(datas = buf.tobytes(), json_data = json.dumps(json_data))
        
    def get_response(self, frame, json_data):
        try:
            responses = self.stub.getStream( self.self_request(frame, json_data))
            for req in responses:
                dBuf = np.frombuffer(req.datas, dtype = np.uint8)
                dst = cv2.imdecode(dBuf, cv2.IMREAD_COLOR)
                res_json_data = json.loads(req.json_data)
                return_dit = {
                    'dst': dst,
                    'res_json_data': res_json_data
                }
                yield return_dit
        except grpc.RpcError as e:
            pass
            
            
if __name__ == '__main__':
    client = Dxr_grpc_client('10.10.8.152', '50051')
    cap = cv2.VideoCapture(0)
    cap.set(cv2.CAP_PROP_FRAME_WIDTH, 640)
    cap.set(cv2.CAP_PROP_FRAME_HEIGHT, 480)
    json_data = {
        'type': 'face',
    }
    while True:
        ret, frame = cap.read()
        if ret != 1:
            continue
        responses = client.get_response(frame, json_data)
        for req in responses:
            cv2.imshow('dst Image', req['dst'])
            k = cv2.waitKey(1)
            if k == 27:
                break
            elif k == ord('g'):
                json_data['type'] = 'gray'
            elif k == ord('b'):
                json_data['type'] = 'blur'
            elif k == ord('c'):
                json_data['type'] = 'canny'
            elif k == ord('r'):
                json_data['type'] = 'rotate'
            elif k == ord('f'):
                json_data['type'] = 'face'
            else:
                pass
            


import asyncio
import edge_tts

VOICE_LIST = [
    'zh-HK-HiuGaaiNeural', 'zh-HK-HiuMaanNeural', 'zh-HK-WanLungNeural',
    'zh-CN-XiaoxiaoNeural', 'zh-CN-XiaoyiNeural', 'zh-CN-YunjianNeural',
    'zh-CN-YunxiNeural', 'zh-CN-YunxiaNeural', 'zh-CN-YunyangNeural',
    'zh-CN-liaoning-XiaobeiNeural', 'zh-TW-HsiaoChenNeural', 'zh-TW-YunJheNeural',
    'zh-TW-HsiaoYuNeural', 'zh-CN-shaanxi-XiaoniNeural'
]

def text_to_speech_sync(text: str, voice_index: int = 3, rate: str = '', voice_file=None) -> str:
    """
    将指定的文本转换为语音，并返回生成的语音文件路径。
    :param text: 要转换为语音的文本内容。
    :param voice_index: 使用的语音类型的索引，默认为3，即'zh-CN-XiaoxiaoNeural'。
    :param rate: 语音的速率，默认为空字符串。
    :return: 生成的语音文件路径。
    """
    voice = VOICE_LIST[voice_index]
    if voice_file is None:
        output_file = f'{voice}.mp3'
    else:
        output_file = voice_file
    communicate = edge_tts.Communicate(text, voice, rate=rate)
    asyncio.run(communicate.save(output_file))
    print(f'语音合成成功，保存路径为：{output_file}，语音类型为：{voice}')
    return output_file

async def text_to_speech_async(text: str, voice_index: int = 3, rate: str = '', voice_file=None) -> str:
    """
    将指定的文本转换为语音，并返回生成的语音文件路径。
    :param text: 要转换为语音的文本内容。
    :param voice_index: 使用的语音类型的索引，默认为3，即'zh-CN-XiaoxiaoNeural'。
    :param rate: 语音的速率，默认为空字符串。
    :return: 生成的语音文件路径。
    """
    voice = VOICE_LIST[voice_index]
    if voice_file is None:
        output_file = f'{voice}.mp3'
    else:
        output_file = voice_file
    communicate = edge_tts.Communicate(text, voice, rate=rate)
    await communicate.save(output_file)
    print(f'语音合成成功，保存路径为：{output_file}，语音类型为：{voice}')
    return output_file


async def main_async(text: str, voice_index: int = 3, rate: str = '') -> None:
    output_file = await text_to_speech_async(text, voice_index, rate, 'test.mp3')
    print(f'生成的语音文件路径为：{output_file}')

def main_sync(text: str, voice_index: int = 3, rate: str = '') -> None:
    output_file = text_to_speech_sync(text, voice_index, rate)
    print(f'生成的语音文件路径为：{output_file}')

if __name__ == '__main__':
    text = '不能否认，微软Azure在TTS(text-to-speech文字转语音)这个人工智能细分领域的影响力是统治级的。'
    voice_index = 3
    rate = '+50%'

    # 异步调用示例
    asyncio.run(main_async(text, voice_index, rate))

    # 同步调用示例
    main_sync(text, voice_index, rate)


import re

from prompt_toolkit import prompt
from prompt_toolkit import PromptSession
from prompt_toolkit.auto_suggest import AutoSuggestFromHistory
from prompt_toolkit.completion import WordCompleter
from prompt_toolkit.history import InMemoryHistory


def create_session() -> PromptSession:
    return PromptSession(history=InMemoryHistory())


def create_completer(commands: list, pattern_str: str = "$") -> WordCompleter:
    return WordCompleter(words=commands, pattern=re.compile(pattern_str))


def get_input(session: PromptSession = None, completer: WordCompleter = None) -> str:
    """
    Multiline input function.
    """
    return (
        session.prompt(
            completer=completer,
            multiline=True,
            auto_suggest=AutoSuggestFromHistory(),
        )
        if session
        else prompt(multiline=True)
    )



"""
hikvision.error
~~~~~~~~~~~~~~~~~~~~

Module errors and exceptionss

Copyright (c) 2015 Finbarr Brady <https://github.com/fbradyirl>
Licensed under the MIT license.
"""


class MissingParamError(Exception):

    """
    This exception is raised when there is a Missing
    Required param.
    """

    def __init__(self, message='', original=None):
        Exception.__init__(self)
        self.message = message
        self.original = original

    def __str__(self):
        if self.original:
            original_name = type(self.original).__name__
            message = '%s Original exception: '\
                '%s, "%s"' % (self.message, original_name, str(self.original))
            return message
        return self.message


class HikvisionError(Exception):

    """
    This exception is raised when there has occurred an error related to
    communication with hikvision. It is a subclass of Exception.
    """

    def __init__(self, message='', original=None):
        Exception.__init__(self)
        self.message = message
        self.original = original

    def __str__(self):
        if self.original:
            original_name = type(self.original).__name__
            message = '%s Original exception:'\
                ' %s, "%s"' % (self.message, original_name, str(self.original))
            return message
        return self.message



# カメラ映像を接続されたクライアントに送信する

# ============================================================
# import packages
# ============================================================
import queue
import threading
from concurrent import futures
import grpc
from . import Datas_pb2
from . import Datas_pb2_grpc
import time
import cv2
import base64
import sys
from . import global_values as gv

# ============================================================
# property
# ============================================================
# カメラを設定


captureBuffer = None
captureBuffer_push_server = None
push_queue = queue.Queue(maxsize=1)


# ============================================================
# class
# ============================================================
# サーバークラス
class Greeter(Datas_pb2_grpc.MainServerServicer):

    # ==========
    def __init__(self):
        pass

    # ==========
    def getStream(self, request_iterator, context):

        for req in request_iterator:

            # リクエストメッセージを表示
            print("request message = ", req.msg)

            while True:
                ret, buf = cv2.imencode('.jpg', captureBuffer)
                if ret != 1:
                    return

                # データを送信
                yield Datas_pb2.Reply(datas=buf.tobytes())

                # 60FPSに設定
                time.sleep(1 / 60)


# ============================================================
# class
# ============================================================
# サーバークラス
class Greeter_push_server(Datas_pb2_grpc.MainServerServicer):

    # ==========
    def __init__(self):
        pass

    # ==========
    def getStream(self, request_iterator, context):

        for req in request_iterator:

            # リクエストメッセージを表示
            print("request message = ", req.msg)

            while True:
                global captureBuffer_push_server
                # print(captureBuffer_push_server)
                ret, buf = cv2.imencode('.jpg', captureBuffer_push_server)
                if ret != 1:
                    return

                # データを送信
                yield Datas_pb2.Reply(datas=buf.tobytes())

                # 60FPSに設定
                time.sleep(1 / 60)


# ============================================================
# functions
# ============================================================
def serve():
    cap = cv2.VideoCapture(gv.url)
    cap.set(cv2.CAP_PROP_FRAME_WIDTH, 640)
    cap.set(cv2.CAP_PROP_FRAME_HEIGHT, 480)

    # サーバーを生成
    server = grpc.server(futures.ThreadPoolExecutor(max_workers=10))
    Datas_pb2_grpc.add_MainServerServicer_to_server(Greeter(), server)

    # ポートを設定
    server.add_insecure_port('[::]:' + str(gv.port))

    # 動作開始
    server.start()

    print('server start')

    while True:
        try:
            # カメラ映像から読み込み
            ret, frame = cap.read()
            if ret != 1:
                continue

            global captureBuffer
            captureBuffer = frame
            time.sleep(0)
            if not gv.server_isStart:
                break

        except KeyboardInterrupt:
            server.stop(0)


def set_url(url):
    gv.url = url


def setPort(port):
    gv.port = port


def start_server():
    gv.server_isStart = True
    threading.Thread(target=serve, daemon=True).start()


def stop_server():
    gv.server_isStart = False


def start_push_server_thread(port):
    gv.push_server_isStart = True
    threading.Thread(target=start_push_server, args=(port,), daemon=True).start()


def stop_push_server_thread():
    gv.push_server_isStart = False


def start_push_server(port):
    # サーバーを生成
    server = grpc.server(futures.ThreadPoolExecutor(max_workers=10))
    Datas_pb2_grpc.add_MainServerServicer_to_server(Greeter_push_server(), server)
    # ポートを設定
    server.add_insecure_port('[::]:' + str(port))

    # 動作開始
    server.start()

    print('push server start')
    while True:
        try:
            # カメラ映像から読み込み
            global push_queue
            frame = push_queue.get(timeout=0.5)
            global captureBuffer_push_server
            captureBuffer_push_server = frame
            time.sleep(0)
            if not gv.push_server_isStart:
                break

        except KeyboardInterrupt:
            server.stop(0)
        except Exception as ex:
            print(ex)
            continue


def push_frame(frame):
    global captureBuffer_push_server, push_queue
    push_queue.queue.clear()
    push_queue.put(frame)
    # print(captureBuffer_push_server)
    pass




# coding=utf-8

from ctypes import *
import sys

# 回调函数类型定义
if 'linux' in sys.platform:
    fun_ctype = CFUNCTYPE
else:
    fun_ctype = WINFUNCTYPE

# 定义预览参数结构体
class FRAME_INFO(Structure):
    pass
LPFRAME_INFO = POINTER(FRAME_INFO)
FRAME_INFO._fields_ = [
    ('nWidth', c_uint32),
    ('nHeight', c_uint32),
    ('nStamp', c_uint32),
    ('nType', c_uint32),
    ('nFrameRate', c_uint32),
    ('dwFrameNum', c_uint32)
]

# 显示回调函数
DISPLAYCBFUN = fun_ctype(None, c_long, c_char_p, c_long, c_long, c_long, c_long, c_long, c_long)
# 解码回调函数
DECCBFUNWIN = fun_ctype(None, c_long, POINTER(c_char), c_long, POINTER(FRAME_INFO), c_void_p, c_void_p)




"""
hikvision.constants
~~~~~~~~~~~~~~~~~~~~

List of constants

Copyright (c) 2015 Finbarr Brady <https://github.com/fbradyirl>
Licensed under the MIT license.
"""

DEFAULT_PORT = None
XML_ENCODING = 'UTF-8'

# Motion Detection
# 1 = 20%
# 4 = 80%
DEFAULT_SENS_LEVEL = 1

DEFAULT_HEADERS = {
    'Content-Type': "application/xml; charset='UTF-8'",
    'Accept': "*/*"
}



from concurrent import futures
import time
import grpc
from . import audios_pb2
from . import audios_pb2_grpc


class Greeter(audios_pb2_grpc.MainServerServicer):
    def __init__(self, rec_callback=None):
        self.rec_callback = rec_callback
    
    def getStream(self, request_iterator, context):
        for request in request_iterator:
            buf = request.datas
            if self.rec_callback:
                buf = self.rec_callback(buf)
            yield audios_pb2.Reply(datas = buf)



class Dxr_grpc_server:
    def __init__(self, port=50051, rec_callback=None) -> None:
        self.port = port
        self.rec_callback = rec_callback

    def serve(self):
        server = grpc.server(futures.ThreadPoolExecutor(max_workers=10))
        audios_pb2_grpc.add_MainServerServicer_to_server(Greeter(self.rec_callback), server)
        server.add_insecure_port('[::]:'+str(self.port))
        server.start()
        print('server start')
        try:
            while True:
                time.sleep(1/60)

        except KeyboardInterrupt:
            server.stop(0)
            
    def run(self):
        self.serve()


from setuptools import setup

setup(
    name='DXR',
    version='1.8.1',
    packages=['Dxr_mqtt', 'Dxr_log', 'Dxr_bytes', 'Dxr_utils', 'Dxr_video', 'Dxr_serial', 'Dxr_yaml', 'Dxr_file', 'Dxr_grpc', 'Dxr_isapi', 'Dxr_voice', 'Dxr_Chat', 'dxr_cli'],
    install_requires=['paho-mqtt', 'pyyaml', 'pyserial', 'loguru','tabulate', 'pymysql', 'sqlalchemy', 'oss2', 'imagezmq', 'simplejpeg', 'pexpect', 'aiortsp'],
    author='luzhipeng',
    author_email='402087139@qq.com',
    license='MIT',
    url='http://pycn.me',
    description='DXR is a python library for DXR_mqtt',
    entry_points={
        'console_scripts': [
            'dxr=dxr_cli.cli:dxr',
        ],
    },
)



from Dxr_log.log import *
import yaml
import re


# 读取批量参数
def read_yaml_file(file):
    """ 读取批量参数
    :param file: 目标yaml文件完整路径
    :return: data 目标yaml文件内容以Dict对象返回
    """
    # 分割路径名称，提取文件对象
    target = re.split(r'[/,\\]', file)[-1]
    # 打开文件
    print_info("start read " + target)
    fd = open(file, "r", encoding="utf-8")
    data_str = fd.read()
    fd.close()
    # yaml读取
    try:
        data = yaml.load(data_str, Loader=yaml.FullLoader)
    except Exception as e:
        if e.__str__().__contains__("Loader"):
            data = yaml.load(data_str)
        else:
            print(e)
    print_info("finish read " + target)
    return data


# 获取对象-通过目标名字（仅支持第一级目标）
def get_value_by_target_name(file, name):
    """ 获取对象-通过目标名字（仅支持第一级目标）
    :param1 file: 目标yaml文件完整路径
    :param2 name: 目标对象名称
    :return: 目标yaml文件中目标对象值，无匹配{}
    """
    # 分割路径名称，提取文件对象
    target = re.split(r'[/,\\]', file)[-1]
    # 打开文件
    print_info("start read " + target)
    fd = open(file, "r", encoding="utf-8")
    data_str = fd.read()
    fd.close()
    # yaml读取
    try:
        data = yaml.load(data_str, Loader=yaml.FullLoader)
    except Exception as e:
        if e.__str__().__contains__("Loader"):
            data = yaml.load(data_str)
        else:
            print(e)
    print_info("finish read " + target)
    # 读取键值为name
    if name in data.keys():
        print("[" + name + "]" + str(data[name]))
        return data[name]
    else:
        print_error("config file do not has " + name)
        return {}


# 更新对象-通过目标对象（仅支持第一级目标）
def update_object_by_target_object(target, data):
    """ 更新对象-通过目标对象（仅支持第一级目标）
    :param1 target: 更新对象
    :param2 data: 目标对象
    """
    if target:
        for x in target.keys():
            if x in data.keys():
                if type(target[x]) == type(data[x]):
                    target[x] = data[x]
                    print("[" + x + "]" + str(data[x]))
                else:
                    print_error("[" + x + "]" + "data type error")
            else:
                print_error("config file do not has " + x)
    else:
        print_error("target is empty")


# 更新部分文件内容-通过目标对象（仅支持第一级目标）
def update_file_by_target_object(file, target_object, target_object_name):
    """ 更新部分文件内容-通过目标对象（仅支持第一级目标）
    :param1 file: 目标yaml文件完整路径
    :param2 target_object: 目标对象
    :param3 target_object_name: 目标对象在文件中的名称
    :return: 更新文件是否成功
    """
    # 分割路径名称，提取文件对象
    target = re.split(r'[/,\\]', file)[-1]
    # 打开文件
    print_info("start read " + target)
    fd = open(file, "r", encoding="utf-8")
    data_str = fd.read()
    print_info("finish read " + target)
    fd.close()
    # yaml读取
    try:
        data = yaml.load(data_str, Loader=yaml.FullLoader)
    except Exception as e:
        if e.__str__().__contains__("Loader"):
            data = yaml.load(data_str)
        else:
            print(e)
    # yaml文件内容与目标对象匹配
    result = True
    if target_object_name in data.keys():
        if type(target_object) is type(data):
            for x in target_object.keys():
                if x in data[target_object_name].keys():
                    if type(target_object[x]) != type(data[target_object_name][x]):
                        result = False
                        print_error("[" + x + "]" + "data type error, save failed")
                        break
                else:
                    result = False
                    print_error("target_object do not has " + x)
                    break
        else:
            if target_object_name in data.keys():
                if type(target_object) != type(data[target_object_name]):
                    result = False
                    print_error("[" + target_object_name + "]" + "data type error, save failed")
    else:
        result = False
        print_error("config file do not has " + target_object_name)
    # 匹配成功，写入target_object
    if result:
        data[target_object_name] = target_object
        fd_1 = open(file, "w", encoding="utf-8")
        str_temp = yaml.dump(data, allow_unicode=True)
        fd_1.write(str_temp)
        fd_1.close()
        return True
    # 失败返回
    else:
        return False


# 更新整个文件内容-通过目标对象
def update_file_all(file, target_object):
    """ 更新整个文件内容-通过目标对象
    :param1 file: 目标yaml文件完整路径
    :param2 target_object: 写入对象
    :return: 更新文件是否成功
    """
    try:
        target = re.split(r'[/,\\]', file)[-1]
        print_info("start open " + target)
        fd = open(file, "w", encoding="utf-8")
        str_temp = yaml.dump(target_object, allow_unicode=True)
        fd.write(str_temp)
        fd.close()
        return True
    except:
        return False


# 测试代码
if __name__ == '__main__':
    # 读取yaml文件
    config_test = read_yaml_file("./config_test.yaml")
    print("config_test:" + str(config_test))
    com_config_set = {
        "port_name": "COM5",
        "bps": 9600,
        "parity": "N"
    }
    # 获取对象-通过目标名字
    com_config_read = get_value_by_target_name("./config_test.yaml", "com_config")
    print("com_config_read:" + str(com_config_read))
    # 更新对象-通过目标对象
    update_object_by_target_object(com_config_read, com_config_set)
    print("com_config_read update:" + str(com_config_read))
    com_config_set = {
        "port_name": "COM3",
        "bps": 115200,
        "parity": "N"
    }
    # 更新部分文件内容-通过目标对象
    print(str(update_file_by_target_object("./config_test.yaml", com_config_set, "com_config")))
    print("update_file_by_target_object:" + str(read_yaml_file("./config_test.yaml")))
    # 更新整个文件内容-通过目标对象
    print(str(update_file_all("./config_test.yaml", config_test)))
    print("update_file_all:" + str(read_yaml_file("./config_test.yaml")))



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: audios.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x0c\x61udios.proto\"\x18\n\x07Request\x12\r\n\x05\x64\x61tas\x18\x01 \x01(\x0c\"\x16\n\x05Reply\x12\r\n\x05\x64\x61tas\x18\x01 \x01(\t21\n\nMainServer\x12#\n\tgetStream\x12\x08.Request\x1a\x06.Reply\"\x00(\x01\x30\x01\x62\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'audios_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _REQUEST._serialized_start=16
  _REQUEST._serialized_end=40
  _REPLY._serialized_start=42
  _REPLY._serialized_end=64
  _MAINSERVER._serialized_start=66
  _MAINSERVER._serialized_end=115
# @@protoc_insertion_point(module_scope)



import json
import grpc
import cv2
import numpy as np
from . import audios_pb2
from . import audios_pb2_grpc
from threading import Thread
import time
            
class Dxr_grpc_client:
    def __init__(self, ip, port):
        self.ip = ip
        self.port = port
        self.channel = grpc.insecure_channel(self.ip + ':' + self.port)
        self.stub = audios_pb2_grpc.MainServerStub(self.channel)
        
    def _keep_connect(self):
        while True:
            time.sleep(1)
            # grpc_health.v1.Health.Check
            res = grpc.channel_ready_future(self.channel).result()
            if res is not None:
                print('channel is not ready, reconnecting...')
                self.channel = grpc.insecure_channel(self.ip + ':' + self.port)
                self.stub = audios_pb2_grpc.MainServerStub(self.channel)
            else:
                pass
            
    def self_request(self, data):
        yield audios_pb2.Request(datas = data)
        
    def get_response(self, data):
        try:
            responses = self.stub.getStream( self.self_request(data))
            for req in responses:
                data = req.datas
                yield data
        except grpc.RpcError as e:
            pass
            
            
if __name__ == '__main__':
    client = Dxr_grpc_client('10.10.8.152', '50051')
    cap = cv2.VideoCapture(0)
    cap.set(cv2.CAP_PROP_FRAME_WIDTH, 640)
    cap.set(cv2.CAP_PROP_FRAME_HEIGHT, 480)
    json_data = {
        'type': 'face',
    }
    while True:
        ret, frame = cap.read()
        if ret != 1:
            continue
        responses = client.get_response(frame, json_data)
        for req in responses:
            cv2.imshow('dst Image', req['dst'])
            k = cv2.waitKey(1)
            if k == 27:
                break
            elif k == ord('g'):
                json_data['type'] = 'gray'
            elif k == ord('b'):
                json_data['type'] = 'blur'
            elif k == ord('c'):
                json_data['type'] = 'canny'
            elif k == ord('r'):
                json_data['type'] = 'rotate'
            elif k == ord('f'):
                json_data['type'] = 'face'
            else:
                pass
            